define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/colorpicker-test',
    'jquery',
    'ep/ui/colorpicker'
], function (QUnit, fixture, $, ep) {

    QUnit.module("ep/ui/colorpicker", {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            '_init',
            '_build',
            '_setOption',
            '_setActive',
            '_colorSelect',
            '_colorApply',
            'colorSelect',
            'colorApply',
            'colorReset',
            'open',
            'cancel',
            'close',
            'destroy'
        ], function (i, fn) {
                QUnit.equal(
                $.isFunction($.ui.uiColorpicker.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("ep/ui/colorpicker (base layout)", function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var t,

            pickerA = ep('#colorpickerA')
                .css('background-color', '#fff')
                .on('colorApply', function (event, color) {
                $(this).css('background-color', color);
            })
                .uiColorpicker({
                orientationAdjust: [10, 0],
                color: 'red'
            }),

            pickerB = ep('#colorpickerB')
                .on('colorSelect', function (event, color) {
                $(this).css('background-color', color);
            })
                .on('colorApply', function (event, color) {
                $(this).css('background-color', color);
            })
                .uiColorpicker({
                orientationAdjust: [10, 0],
                color: 'blue',
                autoOpen: true
            }),

            pickerC = ep('#colorpickerC')
                .on('colorApply', function (event, color) {
                $(this).css('background-color', color);
            })
                .uiColorpicker({
                orientationAdjust: [10, 0],
                color: 'yellow',
                autoApply: true
            }),

            pickerD = ep('#colorpickerD')
                .on('colorApply', function (event, color) {
                $(this).css('background-color', color);
            })
                .uiColorpicker({
                extended: {
                    userColors: {
                        'UserDefinedColor1': '#0a528f',
                        'UserDefinedColor2': '#eb670b',
                        'UserDefinedColor3': '#b0bd35',
                        'UserDefinedColor4': '#7336f0',
                        'UserDefinedColor5': '#59506f',
                        'UserDefinedColor6': '#d3ec5a',
                        'UserDefinedColor7': '#6bdc5c',
                        'UserDefinedColor8': '#1d9675',
                        'UserDefinedColor9': '#e087C2',
                        'UserDefinedColor10': '#47b03d',
                        'UserDefinedColor11': '#8d6319',
                        'UserDefinedColor12': '#bfd33c',
                        'UserDefinedColor13': '#85ec28',
                        'UserDefinedColor14': '#5ad9e0',
                        'UserDefinedColor15': '#b27e86',
                        'UserDefinedColor16': '#420b62',
                        'UserDefinedColor17': '#f7c93f',
                        'UserDefinedColor18': '#60fbfc',
                        'UserDefinedColor19': '#85bbc6'
                    },
                    baseColors: 'rgb(0,128,0)',
                    userColorsSave: function (data) {
                        t = data;
                    }
                },
                orientationAdjust: [10, 0],
                color: 'green'
            });

        QUnit.equal(
            $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length,
            1,
            "*autoOpen* option of *pickerB*: $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length");

        pickerB.uiColorpicker('close');

        QUnit.equal(
            $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length,
            0,
            "*autoOpen* option of *pickerB*: $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length");

        QUnit.equal(
            pickerA.css('background-color'),
            'rgb(255, 255, 255)',
            "*background-color* of *pickerA* button was not changed by *pickerA* at start-up: pickerA.css('background-color')");

        QUnit.equal(
            pickerB.css('background-color'),
            'rgb(0, 0, 255)',
            "*pickerB* fired *colorSelect* event: pickerB.css('background-color')");

        pickerB
            .off('colorSelect')
            .uiColorpicker('colorSelect', 'green')
            .uiColorpicker('colorApply');

        QUnit.equal(
            pickerB.css('background-color'),
            "rgb(0, 128, 0)",
            "Apply selected color by public widget method *colorApply*");

        pickerB
            .uiColorpicker('colorSelect', 'red');

        QUnit.equal(
            pickerB.uiColorpicker('option', 'color'),
            "red",
            "Set color by public widget method *colorSelect*");

        pickerB
            .uiColorpicker('colorSelect', 'red')
            .uiColorpicker('colorReset');

        QUnit.equal(
            pickerB.uiColorpicker('option', 'color'),
            "green",
            "Reset color by public widget method *colorReset*");

        pickerC.uiColorpicker('open');

        QUnit.equal(
            pickerC.css('background-color'),
            'rgb(255, 255, 0)',
            "*pickerC* fired *colorApply* event: pickerC.css('background-color')");

        QUnit.equal(
            $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length,
            1,
            "*open* method of *pickerC*: $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length,");

        var $preview = $('.ep-uiSimpledialog .ep-uiColorpicker-preview').filter(':visible');

        QUnit.equal(
            $preview.css('background-color'),
            "rgb(255, 255, 0)",
            "*pickerC*'s preview shows the right color: $preview.css('background-color')");

        pickerD.trigger('click');

        QUnit.equal(
            $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length,
            1,
            "Clicking *pickerD* button closed other picker dialogs and opened a new one: $('.ep-uiSimpledialog .ep-uiColorpicker-box').filter(':visible').length");

        $preview = $('.ep-uiSimpledialog .ep-uiColorpicker-preview').filter(':visible');

        QUnit.equal(
            $preview.css('background-color'),
            "rgb(0, 128, 0)",
            "*pickerD*'s preview shows the right color: $preview.css('background-color')");

        pickerD.uiColorpicker('colorSelect', 'blue');

        QUnit.equal(
            $preview.css('background-color'),
            "rgb(0, 0, 255)",
            "*colorSelect* method of *pickerD* works (UI): $preview.css('background-color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            "blue",
            "*colorSelect* method of *pickerD* works (Data): pickerD.uiColorpicker('option','color')");

        var $checkbox = $('.ep-uiSimpledialog .ep-uiColorpicker-buttons input[type=checkbox]').filter(':visible');

        QUnit.equal(
            $checkbox.prop('checked'),
            false,
            "*transparentInput* checkbox of *pickerD* is not checked (a color has been selected): $transparentInput.prop('checked')");

        var $originalBox = $('.ep-uiSimpledialog .ep-uiColorpicker-original').filter(':visible');
        $originalBox.trigger({
            'type': 'click',
            'originalTarget': $originalBox[0]
        });

        QUnit.equal(
            $preview.css('background-color'),
            'rgb(0, 128, 0)',
            "Clicking *originalBox* restores original color (UI): $preview.css('background-color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'green',
            "Clicking *originalBox* restores original color (Data): $preview.css('background-color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'green',
            "Clicking *originalBox* restores original color (Data): $preview.css('background-color')");

        pickerD.uiColorpicker('option', 'transparent', false);
        pickerD.data('uiUiColorpicker')
            .hexInput
            .val('transparent')
            .trigger('change');

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'white',
            "Disable *transparant* option and try typing transparent into text input");

        pickerD.uiColorpicker('option', 'transparent', true);

        var $firstUserColor = $('.ep-uiColorpicker-colors:visible').first().children('li:first');
        $firstUserColor.trigger({
            'type': 'click',
            'originalTarget': $firstUserColor[0]
        });

        QUnit.equal(
            $preview.css('background-color'),
            $firstUserColor.css('background-color'),
            "Clicked first of *userColors* (UI):  $preview.css('background-color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            '#0a528f',
            "Clicked first of *userColors* (Data): pickerD.uiColorpicker('option','color')");

        var $firstBaseColor = $($('.ep-uiColorpicker-colors:visible')[1]).children('li:first');
        $firstBaseColor.trigger({
            'type': 'click',
            'originalTarget': $firstBaseColor[0]
        });

        QUnit.equal(
            $preview.css('background-color'),
            $firstBaseColor.css('background-color'),
            "Clicked first of *baseColors* (UI): $preview.css('background-color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'green',
            "Clicked first of *baseColors* (Data): pickerD.uiColorpicker('option','color')");

        var $transparentInput_span = $checkbox.parent().siblings();
        $checkbox.trigger({
            'type': 'click',
            'originalTarget': $transparentInput_span[0]
        });

        QUnit.equal(
            $preview.css('background-color'),
            "rgb(255, 255, 255)",
            "Clicked *transparentInput* span (UI-1): $preview.css('background-color')");

        QUnit.equal(
            $checkbox.prop('checked'),
            true,
            "Clicked *transparentInput* span (UI-2): $transparentInput.prop('checked')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'transparent',
            "Clicked *transparentInput* span (Data): pickerD.uiColorpicker('option', 'color')");

        var $canvas = $('.ep-uiSimpledialog canvas.ep-colorpicker').filter(':visible');
        $canvas.trigger({
            'type': 'mousedown',
            'originalTarget': $canvas[0]
        });

        QUnit.equal(
            $checkbox.prop('checked'),
            false,
            "Click on canvas unchecks *transparentInput* checkbox: $transparentInput.prop('checked')");

        $('.ep-uiSimpledialog .ep-uiColorpicker-hexInput').filter(':visible')
            .val('white')
            .trigger('change');

        QUnit.equal(
            pickerD.uiColorpicker('option', 'color'),
            'white',
            "Changed value of *hexInput*: pickerD.uiColorpicker('option','color')");

        QUnit.equal(
            pickerD.uiColorpicker('option', 'transparent'),
            true,
            "By default *transparent* option is true: pickerD.uiColorpicker('option', 'transparent')");

        pickerD.uiColorpicker('option', 'transparent', false);

        QUnit.equal(
            pickerD.uiColorpicker('option', 'transparent'),
            false,
            "Set *transparent* option: pickerD.uiColorpicker('option', 'transparent')");

        var $userColor = $('.ep-uiSimpledialog .ep-uiColorpicker-colors li:eq(2)').filter(':visible');
        $userColor.trigger({
            type: 'click',
            originalTarget: $userColor[0]
        });

        QUnit.ok(
            $('.ep-uiColorpicker-dialog:visible .active')[0] === $('.ep-uiColorpicker-dialog:visible .ep-uiColorpicker-colors li:eq(2)')[0],
            "Seleced a user defined color by click, current active element is this user defined color.");

        $userColor.trigger({
            type: 'dblclick',
            originalTarget: $userColor[0]
        });

        QUnit.ok(
            $('.ep-uiColorpicker-dialog:visible .active')[0] === $('.ep-uiColorpicker-dialog:visible .ep-uiColorpicker-preview')[0],
            "Seleced a user defined color by double click, current active element is the preview box.");

        $applyButton = $('button.ep-uiColorpicker-applyButton:visible');
        $applyButton.trigger({
            'type': 'click',
            'originalTarget': $applyButton[0]
        });

        QUnit.equal(
            pickerD.css('background-color'),
            "rgb(176, 189, 53)",
            "Clicked *applyButton*, so *colorApply* event was fired: pickerD.css('background-color')");

        pickerC.uiColorpicker('open');

        setTimeout(function () {
            $('body').trigger('click');

            QUnit.equal(
                $('.ep-uiColorpicker-dialog:visible').length,
                0,
                "Clicking anywhere outside the dialog, closes the dialog: $('.ep-uiColorpicker-dialog:visible').length");

            QUnit.deepEqual(
                t, [{
                    "name": "UserDefinedColor1",
                    "value": "white"
                }, {
                    "name": "UserDefinedColor2",
                    "value": "#eb670b"
                }, {
                    "name": "UserDefinedColor3",
                    "value": "#b0bd35"
                }, {
                    "name": "UserDefinedColor4",
                    "value": "#7336f0"
                }, {
                    "name": "UserDefinedColor5",
                    "value": "#59506f"
                }, {
                    "name": "UserDefinedColor6",
                    "value": "#d3ec5a"
                }, {
                    "name": "UserDefinedColor7",
                    "value": "#6bdc5c"
                }, {
                    "name": "UserDefinedColor8",
                    "value": "#1d9675"
                }, {
                    "name": "UserDefinedColor9",
                    "value": "#e087C2"
                }, {
                    "name": "UserDefinedColor10",
                    "value": "#47b03d"
                }, {
                    "name": "UserDefinedColor11",
                    "value": "#8d6319"
                }, {
                    "name": "UserDefinedColor12",
                    "value": "#bfd33c"
                }, {
                    "name": "UserDefinedColor13",
                    "value": "#85ec28"
                }, {
                    "name": "UserDefinedColor14",
                    "value": "#5ad9e0"
                }, {
                    "name": "UserDefinedColor15",
                    "value": "#b27e86"
                }, {
                    "name": "UserDefinedColor16",
                    "value": "#420b62"
                }, {
                    "name": "UserDefinedColor17",
                    "value": "#f7c93f"
                }, {
                    "name": "UserDefinedColor18",
                    "value": "#60fbfc"
                }, {
                    "name": "UserDefinedColor19",
                    "value": "#85bbc6"
                }],
                "Saved *userColors* using *extended.userColorsSave* option");

            var dialog_count = $('.ep-uiSimpledialog .ep-uiColorpicker-box').length;

            pickerD.uiColorpicker('destroy');

            QUnit.equal(
                dialog_count - $('.ep-uiSimpledialog .ep-uiColorpicker-box').length,
                1,
                "Destroyed dialog: dialog_count - $('.ep-uiSimpledialog .ep-uiColorpicker-box').length");

            QUnit.start();
        }, 1);

    });


    QUnit.asyncTest("ep/ui/colorpicker (scribe layout)", function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var colorpicker = $('#colorpicker_scribe'),
            layout = 'scribe',
            newColor = '#abcdef';

        colorpicker.uiColorpicker({
            layout: layout,
            color: 'red',
            callback: function(color) {
                QUnit.equal(
                    color,
                    null,
                    "Reset color button was pressed."
                );
            }
        });

        window.setTimeout(function() {
            QUnit.ok(
                colorpicker.data('uiUiColorpicker') !== undefined,
                "Colorpicker for scribe was initiated."
            );

            colorpicker.uiColorpicker('open');

            window.setTimeout(function() {
                QUnit.equal(
                    colorpicker.data('uiUiColorpicker').element.uiSimpledialog('option', 'layout'),
                    layout,
                    "Colorpicker was rendered with scribe layout."
                );

                QUnit.ok(
                    !!colorpicker.data('uiUiColorpicker').resetButton,
                    "Colorpicker dialog contains a reset button."
                );
                QUnit.equal(
                    colorpicker.data('uiUiColorpicker').hexInput.val(),
                    "red",
                    "Initial color 'red' was inserted in input field."
                );

                // triggers test in callback function
                colorpicker.data('uiUiColorpicker').resetButton.trigger('click');

                // change callback to test color value
                colorpicker.uiColorpicker('option', 'callback', function(color) {
                    QUnit.equal(
                        color,
                        newColor,
                        "Apply button was pressed and the correct color string '" + newColor + "' is present."
                    );
                });

                // open color picker dialog
                colorpicker.uiColorpicker('open');

                window.setTimeout(function() {
                    // set new color value
                    colorpicker.data('uiUiColorpicker').hexInput.val(newColor);
                    colorpicker.data('uiUiColorpicker').hexInput.trigger('change');
                    colorpicker.data('uiUiColorpicker').applyButton.trigger('click');

                    // destroy widget
                    colorpicker.uiColorpicker('destroy');

                    QUnit.ok(
                        colorpicker.data('uiUiColorpicker') === undefined,
                        "Widget was destroyed."
                    );

                    QUnit.start();
                }, 10);
            }, 300);
        }, 100);
    });

});
